@@ -44,6 +44,7 @@ const Mainloop = imports.mainloop;
 const MimeHandler = imports.ui.mimeHandler;
 const Constants = imports.util.constants;
 const SpinnerBox = imports.ui.spinnerBox;
+const Utils = imports.ui.utils;
 
 const Sushi = imports.gi.Sushi;
 
@@ -70,26 +71,29 @@ MainWindow.prototype = {
         this._createGtkWindow();
         this._createClutterEmbed();
 
-        this._connectStageSignals();
 	this.file = null;
     },
 
     _createGtkWindow : function() {
+        this._settings = new Gio.Settings({ schema_id: 'org.gnome.sushi' });
+        this._clientDecorated = this._settings.get_boolean('client-decoration');
+
         this._gtkWindow = new Gtk.Window({ type: Gtk.WindowType.TOPLEVEL,
                                            focusOnMap: true,
-                                           decorated: false,
+                                           decorated: !this._clientDecorated,
                                            hasResizeGrip: false,
                                            skipPagerHint: true,
                                            skipTaskbarHint: true,
                                            windowPosition: Gtk.WindowPosition.CENTER,
-                                           gravity: Gdk.Gravity.CENTER });
+                                           gravity: Gdk.Gravity.CENTER,
+                                           application: this._application });
 
         let screen = Gdk.Screen.get_default();
         this._gtkWindow.set_visual(screen.get_rgba_visual());
 
-        this._gtkWindow.connect("delete-event",
+        this._gtkWindow.connect('delete-event',
                                 Lang.bind(this, this._onWindowDeleteEvent));
-        this._gtkWindow.connect("realize", Lang.bind(this,
+        this._gtkWindow.connect('realize', Lang.bind(this,
             function() {
                 // don't support maximize and minimize
                 this._gtkWindow.get_window().set_functions(Gdk.WMFunction.MOVE |
@@ -112,17 +116,20 @@ MainWindow.prototype = {
                                                   green: 0,
                                                   blue: 0,
                                                   alpha: 255 }));
-        this._mainGroup =  new Clutter.Group();
+
+        this._mainLayout = new Clutter.BinLayout();
+        this._mainGroup = new Clutter.Actor({ layout_manager: this._mainLayout });
+        this._mainGroup.add_constraint(
+            new Clutter.BindConstraint({ coordinate: Clutter.BindCoordinate.SIZE,
+                                         source: this._stage }));
         this._stage.add_actor(this._mainGroup);
         this._mainGroup.set_opacity(0);
-    },
 
-    _connectStageSignals : function() {
-        this._stage.connect("key-press-event",
+        this._stage.connect('key-press-event',
                             Lang.bind(this, this._onStageKeyPressEvent));
-        this._stage.connect("button-press-event",
+        this._stage.connect('button-press-event',
                             Lang.bind(this, this._onButtonPressEvent));
-        this._stage.connect("motion-event",
+        this._stage.connect('motion-event',
                             Lang.bind(this, this._onMotionEvent));
     },
 
@@ -136,14 +143,8 @@ MainWindow.prototype = {
                                                        green: 0,
                                                        blue: 0,
                                                        alpha: 255 }));
-        this._background.add_constraint(
-            new Clutter.BindConstraint({ source: this._stage,
-                                         coordinate: Clutter.BindCoordinate.POSITION }));
-        this._background.add_constraint(
-            new Clutter.BindConstraint({ source: this._stage,
-                                         coordinate: Clutter.BindCoordinate.SIZE }));
 
-        this._mainGroup.add_actor(this._background);
+        this._mainLayout.add(this._background, Clutter.BinAlignment.FILL, Clutter.BinAlignment.FILL);
         this._background.lower_bottom();
     },
 
@@ -151,16 +152,19 @@ MainWindow.prototype = {
         if (this._background)
             return;
 
-        this._background = Sushi.create_rounded_background();
+        if (this._clientDecorated) {
+            this._background = Sushi.create_rounded_background();
+        } else {
+            this._background = new Clutter.Rectangle();
+            this._background.set_color(new Clutter.Color({ red: 0,
+                                                           green: 0,
+                                                           blue: 0,
+                                                           alpha: 255 }));
+        }
+
         this._background.set_opacity(Constants.VIEW_BACKGROUND_OPACITY);
-        this._background.add_constraint(
-            new Clutter.BindConstraint({ source: this._stage,
-                                         coordinate: Clutter.BindCoordinate.POSITION }));
-        this._background.add_constraint(
-            new Clutter.BindConstraint({ source: this._stage,
-                                         coordinate: Clutter.BindCoordinate.SIZE }));
+        this._mainLayout.add(this._background, Clutter.BinAlignment.FILL, Clutter.BinAlignment.FILL);
 
-        this._mainGroup.add_actor(this._background);
         this._background.lower_bottom();
     },
 
@@ -198,7 +202,7 @@ MainWindow.prototype = {
             return false;
         }
 
-        let root_coords = 
+        let root_coords =
             this._gtkWindow.get_window().get_root_coords(win_coords[0],
                                                          win_coords[1]);
 
@@ -223,9 +227,10 @@ MainWindow.prototype = {
     _getTextureSize : function() {
         let screenSize = [ this._gtkWindow.get_window().get_width(),
                            this._gtkWindow.get_window().get_height() ];
+        let yPadding = this._clientDecorated ? Constants.VIEW_PADDING_Y : 0;
 
         let availableWidth = this._isFullScreen ? screenSize[0] : Constants.VIEW_MAX_W - 2 * Constants.VIEW_PADDING_X;
-        let availableHeight = this._isFullScreen ? screenSize[1] : Constants.VIEW_MAX_H - Constants.VIEW_PADDING_Y;
+        let availableHeight = this._isFullScreen ? screenSize[1] : Constants.VIEW_MAX_H - yPadding;
 
         let textureSize = this._renderer.getSizeForAllocation([availableWidth, availableHeight], this._isFullScreen);
 
@@ -235,13 +240,14 @@ MainWindow.prototype = {
     _getWindowSize : function() {
         let textureSize = this._getTextureSize();
         let windowSize = textureSize;
+        let yPadding = this._clientDecorated ? Constants.VIEW_PADDING_Y : 0;
 
         if (textureSize[0] < (Constants.VIEW_MIN - 2 * Constants.VIEW_PADDING_X) &&
-            textureSize[1] < (Constants.VIEW_MIN - Constants.VIEW_PADDING_Y)) {
+            textureSize[1] < (Constants.VIEW_MIN - yPadding)) {
             windowSize = [ Constants.VIEW_MIN, Constants.VIEW_MIN ];
         } else if (!this._isFullScreen) {
             windowSize = [ windowSize[0] + 2 * Constants.VIEW_PADDING_X,
-                           windowSize[1] + Constants.VIEW_PADDING_Y ];
+                           windowSize[1] + yPadding ];
         }
 
         return windowSize;
@@ -259,7 +265,7 @@ MainWindow.prototype = {
         }
 
         if (yFactor == 0) {
-            if (this._isFullScreen && 
+            if (this._isFullScreen &&
                (textureSize[0] > textureSize[1]))
                 yFactor = 0.52;
             else
@@ -296,7 +302,7 @@ MainWindow.prototype = {
         this._renderer.startTimeout();
 
         file.query_info_async
-        (Gio.FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME + "," +
+        (Gio.FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME + ',' +
          Gio.FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
          Gio.FileQueryInfoFlags.NONE,
          GLib.PRIORITY_DEFAULT, null,
@@ -333,20 +339,14 @@ MainWindow.prototype = {
         }
 
         this._texture = this._renderer.render();
-
-        this._textureXAlign = 
-            new Clutter.AlignConstraint({ source: this._stage,
-                                          factor: 0.5 });
         this._textureYAlign =
             new Clutter.AlignConstraint({ source: this._stage,
                                           factor: 0.5,
-                                          "align-axis": Clutter.AlignAxis.Y_AXIS })
-
-        this._texture.add_constraint(this._textureXAlign);
+                                          align_axis: Clutter.AlignAxis.Y_AXIS });
         this._texture.add_constraint(this._textureYAlign);
 
         this.refreshSize();
-        this._mainGroup.add_actor(this._texture);
+        this._mainGroup.add_child(this._texture);
     },
 
     /**************************************************************************
@@ -370,12 +370,12 @@ MainWindow.prototype = {
         Tweener.addTween(this._mainGroup,
                          { opacity: 255,
                            time: 0.15,
-                           transition: 'easeOutQuad',
+                           transition: 'easeOutQuad'
                          });
         Tweener.addTween(this._titleGroup,
                          { opacity: 255,
                            time: 0.15,
-                           transition: 'easeOutQuad',
+                           transition: 'easeOutQuad'
                          });
     },
 
@@ -387,11 +387,11 @@ MainWindow.prototype = {
             this._removeToolbarTimeout();
         }
 
-        /* wait for the next stage allocation to fade in the texture 
+        /* wait for the next stage allocation to fade in the texture
          * and background again.
          */
         this._unFullScreenId =
-            this._stage.connect("notify::allocation",
+            this._stage.connect('notify::allocation',
                                 Lang.bind(this, this._onStageUnFullScreen));
 
         /* quickly fade out everything,
@@ -458,11 +458,11 @@ MainWindow.prototype = {
             this._removeToolbarTimeout();
         }
 
-        /* wait for the next stage allocation to fade in the texture 
+        /* wait for the next stage allocation to fade in the texture
          * and background again.
          */
         this._fullScreenId =
-            this._stage.connect("notify::allocation",
+            this._stage.connect('notify::allocation',
                                 Lang.bind(this, this._onStageFullScreen));
 
         /* quickly fade out everything,
@@ -500,24 +500,17 @@ MainWindow.prototype = {
         if (!this._toolbarActor)
             return;
 
+        Utils.alphaGtkWidget(this._toolbarActor.get_widget());
+
         this._toolbarActor.set_reactive(true);
         this._toolbarActor.set_opacity(0);
-        this._mainGroup.add_actor(this._toolbarActor);
 
-        this._toolbarActor.add_constraint(
-            new Clutter.AlignConstraint({ source: this._stage,
-                                          factor: 0.5 }));
+        this._toolbarActor.margin_bottom = Constants.TOOLBAR_SPACING;
+        this._toolbarActor.margin_left = Constants.TOOLBAR_SPACING;
+        this._toolbarActor.margin_right = Constants.TOOLBAR_SPACING;
 
-        let yConstraint = 
-            new Clutter.BindConstraint({ source: this._stage,
-                                         coordinate: Clutter.BindCoordinate.Y,
-                                         offset: this._stage.height - Constants.TOOLBAR_SPACING });
-        this._toolbarActor.add_constraint(yConstraint);
-
-        this._stage.connect("notify::height",
-                            Lang.bind(this, function() {
-                                yConstraint.set_offset(this._stage.height - Constants.TOOLBAR_SPACING);
-                            }));
+        this._mainLayout.add(this._toolbarActor,
+                             Clutter.BinAlignment.CENTER, Clutter.BinAlignment.END);
     },
 
     _removeToolbarTimeout: function() {
@@ -537,7 +530,7 @@ MainWindow.prototype = {
             Tweener.addTween(this._toolbarActor,
                              { opacity: 200,
                                time: 0.1,
-                               transition: 'easeOutQuad',
+                               transition: 'easeOutQuad'
                              });
         }
 
@@ -576,25 +569,31 @@ MainWindow.prototype = {
             new Clutter.BindConstraint({ source: this._stage,
                                          coordinate: Clutter.BindCoordinate.WIDTH }));
 
-        this._titleLabel = new Gtk.Label({ label: "",
+        // if we don't draw client decorations, just create an empty actor
+        if (!this._clientDecorated)
+            return;
+
+        this._titleLabel = new Gtk.Label({ label: '',
 					   ellipsize: Pango.EllipsizeMode.END,
                                            margin: 6 });
-        this._titleLabel.get_style_context().add_class("np-decoration");
-        
+        this._titleLabel.get_style_context().add_class('np-decoration');
+
         this._titleLabel.show();
         this._titleActor = new GtkClutter.Actor({ contents: this._titleLabel });
+        Utils.alphaGtkWidget(this._titleActor.get_widget());
 
-        this._quitButton = 
-            new Gtk.Button({ image: new Gtk.Image ({ "icon-size": Gtk.IconSize.MENU,
-                                                     "icon-name": "window-close-symbolic" })});
-        this._quitButton.get_style_context().add_class("np-decoration");
+        this._quitButton =
+            new Gtk.Button({ image: new Gtk.Image ({ icon_size: Gtk.IconSize.MENU,
+                                                     icon_name: 'window-close-symbolic' })});
+        this._quitButton.get_style_context().add_class('np-decoration');
         this._quitButton.show();
 
-        this._quitButton.connect("clicked",
+        this._quitButton.connect('clicked',
                                  Lang.bind(this,
                                            this._clearAndQuit));
 
         this._quitActor = new GtkClutter.Actor({ contents: this._quitButton });
+        Utils.alphaGtkWidget(this._quitActor.get_widget());
         this._quitActor.set_reactive(true);
 
         let hidden = new Clutter.Actor();
@@ -634,7 +633,7 @@ MainWindow.prototype = {
         Tweener.addTween(this._titleGroup,
                          { opacity: 0,
                            time: 0.15,
-                           transition: 'easeOutQuad',
+                           transition: 'easeOutQuad'
                          });
 
         Tweener.addTween(this._mainGroup,
@@ -652,7 +651,7 @@ MainWindow.prototype = {
         if (this._renderer.clear)
             this._renderer.clear();
 
-        this._application.quit();
+        this._gtkWindow.destroy();
     },
 
     /**************************************************************************
@@ -679,7 +678,10 @@ MainWindow.prototype = {
     },
 
     setTitle : function(label) {
-        this._titleLabel.set_label(label);
+        if (this._clientDecorated)
+            this._titleLabel.set_label(label);
+
+        this._gtkWindow.set_title(label);
     },
 
     refreshSize : function() {
