@@ -25,6 +25,6 @@
  *
  */
 
-let LOCALE_DIR = "/usr/local/share/locale";
-let ICONS_DIR = "/usr/local/share/sushi/icons/";
-let STYLE_DIR = "/usr/local/share/sushi/style/";
\ No newline at end of file
+let LOCALE_DIR = '/usr/local/share/locale';
+let ICONS_DIR = '/usr/local/share/sushi/icons/';
+let STYLE_DIR = '/usr/local/share/sushi/style/';
\ No newline at end of file
